
/***************************************************************************
 *  Description:
 *      Contact a server running reflectip to determine our IP address
 *      as seen by that server.  This allows us to find out the outside
 *      address of any NAT gateways we're going through.
 *
 *  Arguments:
 *      1.  hostname of server
 *      2.  TCP port of reflectip daemon on that server
 *
 *  Returns:
 *      See sysexits.h.
 *
 *  History: 
 *  Date        Name        Modification
 *  2017-06-23  Jason Bacon Begin
 ***************************************************************************/

/* SunOS Compile: cc prog -lsocket -lnsl */
/* SCO_SV Compile: cc -v prog -lsocket */

/* client.c */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
/* Place sysexits after unistd to work around a bug in IRIX.  EX_OK
   is defined differently in these two headers, and we want the
   sysexists version to override here */
#include <sysexits.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

/* Enough for IPv4 address */
#define ADDR_STR_LEN    16

/***************************************************************************
 *  Description:
 *      Resolve a hostname to an IP address.
 *
 *  Arguments:
 *      1.  Fully qualified hostname
 *      2.  Buffer to receive IP address
 *      3.  Length of buffer
 *
 *  Returns:
 *      0 on success, non-zero error code otherwise
 *
 *  History: 
 *  Date        Name        Modification
 *  2017-06-23  Jason Bacon Begin
 ***************************************************************************/

int     resolve_host(char * hostname , char* ip, size_t max_ip_len)
{
    struct hostent  *hostent;
    struct in_addr  **addr_list;
    int             c;
	 
    if ( (hostent = gethostbyname( hostname )) == NULL) 
    {
	herror("gethostbyname");
	return EX_NOHOST;
    }
 
    addr_list = (struct in_addr **) hostent->h_addr_list;
     
    for(c = 0; addr_list[c] != NULL; c++) 
    {
	/* Return the first address found */
	strlcpy(ip , inet_ntoa(*addr_list[c]), max_ip_len );
	return 0;
    }
    
    /*
     * Something is wrong if we get this far.  If gethostent() succeeds,
     * we should have found an IP.
     */
    return EX_OSERR;
}


int     main (int argc, char *argv[])
{
    short   tcp_port;   /* Need short for htons() */
    int     fd;
    struct  sockaddr_in server_address;
    char    addr_str[ADDR_STR_LEN+1],
	    *end_ptr;

    if (argc != 3)
    {
	fprintf (stderr, "Usage: %s <hostname> <TCP-port>\n", argv[0]);
	return EX_USAGE;
    }

    /* Get IP address and port from command line */
    if ( resolve_host(argv[1], addr_str, ADDR_STR_LEN) != 0 )
    {
	fprintf(stderr, "Could not resolve %s.\n", argv[1]);
	return EX_DATAERR;
    }
    
    tcp_port = strtol (argv[2], &end_ptr, 10);
    if ( *end_ptr != '\0' )
    {
	fprintf(stderr, "Invalid TCP port %s.\n", argv[2]);
	return EX_DATAERR;
    }

    /* Set up socket structure */
    server_address.sin_family = AF_INET;
    server_address.sin_addr.s_addr = inet_addr (addr_str);
    server_address.sin_port = htons (tcp_port);

    /* Create a socket */
    if ((fd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
    {
	fprintf (stderr, "Error opening socket.\n");
	return EX_NOHOST;
    }

    /* Attempt to connect to server */
    if (connect (fd, (struct sockaddr *)&server_address,
		 sizeof (server_address)) < 0)
    {
	fprintf (stderr, "Error connecting to client.\n");
	return EX_NOHOST;
    }

    /* Read IP response from server */
    read(fd, addr_str, ADDR_STR_LEN);
    puts(addr_str);

    close (fd);
    return EX_OK;
}
